@@ -1,20 +1,6 @@
-/* Copyright (C) 2013-2022 Greenbone Networks GmbH
+/* SPDX-FileCopyrightText: 2013-2023 Greenbone AG
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -48,6 +34,12 @@
  */
 #define G_LOG_DOMAIN "libgvm base"
 
+#if GLIB_CHECK_VERSION(2, 67, 3)
+#define memdup g_memdup2
+#else
+#define memdup g_memdup
+#endif
+
 /* Global variables */
 
 /* Source interface name eg. eth1. */
@@ -81,6 +73,9 @@ gvm_source_iface_init (const char *iface)
   if (iface == NULL)
     return ret;
 
+  if (strlen (iface) >= sizeof (global_source_iface))
+    return ret;
+
   if (getifaddrs (&ifaddr) == -1)
     return ret;
 
@@ -111,7 +106,7 @@ gvm_source_iface_init (const char *iface)
 
   /* At least one address for the interface was found. */
   if (ret == 0)
-    memcpy (global_source_iface, iface, sizeof (global_source_iface) - 1);
+    memcpy (global_source_iface, iface, strlen (iface));
 
   freeifaddrs (ifaddr);
   return ret;
@@ -365,19 +360,13 @@ gvm_resolve_list (const char *name)
         {
           struct sockaddr_in *addrin = (struct sockaddr_in *) p->ai_addr;
           ipv4_as_ipv6 (&(addrin->sin_addr), &dst);
-#pragma GCC diagnostic push
-#pragma GCC diagnostic warning "-Wdeprecated-declarations"
-          list = g_slist_prepend (list, g_memdup (&dst, sizeof (dst)));
-#pragma GCC diagnostic pop
+          list = g_slist_prepend (list, memdup (&dst, sizeof (dst)));
         }
       else if (p->ai_family == AF_INET6)
         {
           struct sockaddr_in6 *addrin = (struct sockaddr_in6 *) p->ai_addr;
           memcpy (&dst, &(addrin->sin6_addr), sizeof (struct in6_addr));
-#pragma GCC diagnostic push
-#pragma GCC diagnostic warning "-Wdeprecated-declarations"
-          list = g_slist_prepend (list, g_memdup (&dst, sizeof (dst)));
-#pragma GCC diagnostic pop
+          list = g_slist_prepend (list, memdup (&dst, sizeof (dst)));
         }
       p = p->ai_next;
     }
