@@ -1,20 +1,6 @@
-/* Copyright (C) 2014-2022 Greenbone Networks GmbH
+/* SPDX-FileCopyrightText: 2014-2023 Greenbone AG
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -40,6 +26,12 @@
  */
 #define G_LOG_DOMAIN "libgvm util"
 
+#if GLIB_CHECK_VERSION(2, 67, 3)
+#define memdup g_memdup2
+#else
+#define memdup g_memdup
+#endif
+
 /**
  * @file kb.c
  *
@@ -692,10 +684,7 @@ redis2kbitem_single (const char *name, const redisReply *elt, int force_int)
   else
     {
       item->type = KB_TYPE_STR;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic warning "-Wdeprecated-declarations"
-      item->v_str = g_memdup (elt->str, elt->len + 1);
-#pragma GCC diagnostic pop
+      item->v_str = memdup (elt->str, elt->len + 1);
       item->len = elt->len;
     }
 
