@@ -44,13 +44,15 @@ typedef struct
 } mqtt_t;
 
 static const char *global_server_uri = NULL;
+static const char *global_username = NULL;
+static const char *global_password = NULL;
 static mqtt_t *global_mqtt_client = NULL;
 static gboolean mqtt_initialized = FALSE;
 
 /**
  * @brief Set the global init status.
 
- * @param Status Status of initialization.
+ * @param status Status of initialization.
  */
 static void
 mqtt_set_initialized_status (gboolean status)
@@ -91,6 +93,46 @@ mqtt_get_global_server_uri ()
   return global_server_uri;
 }
 
+/**
+ * @brief Set the global mqtt username.
+
+ * @param username to set.
+ */
+static void
+mqtt_set_global_username (const char *username)
+{
+  global_username = username;
+}
+
+/**
+ * @brief Get global username.
+ */
+static const char *
+mqtt_get_global_username ()
+{
+  return global_username;
+}
+
+/**
+ * @brief Set the global mqtt password.
+
+ * @param password to set.
+ */
+static void
+mqtt_set_global_password (const char *password)
+{
+  global_password = password;
+}
+
+/**
+ * @brief Get global password.
+ */
+static const char *
+mqtt_get_global_password ()
+{
+  return global_password;
+}
+
 /**
  * @brief
  *
@@ -264,12 +306,16 @@ mqtt_set_client (mqtt_t *mqtt, MQTTClient client)
 /**
  * @brief Make new client and connect to mqtt broker.
  *
- * @param mqtt  Initialized mqtt_t
+ * @param mqtt        Initialized mqtt_t
+ * @param server_uri  Server URI
+ * @param username    Username
+ * @param password    Password
  *
  * @return 0 on success, <0 on error.
  */
 static int
-mqtt_connect (mqtt_t *mqtt, const char *server_uri)
+mqtt_connect (mqtt_t *mqtt, const char *server_uri, const char *username,
+              const char *password)
 {
   int rc;
   MQTTClient client;
@@ -288,6 +334,12 @@ mqtt_connect (mqtt_t *mqtt, const char *server_uri)
   conn_opts.cleanstart = 1;
   conn_opts.MQTTVersion = MQTTVERSION_5;
 
+  if (username != NULL && password != NULL)
+    {
+      conn_opts.username = username;
+      conn_opts.password = password;
+    }
+
   resp = MQTTClient_connect5 (client, &conn_opts, &connect_properties, NULL);
   rc = resp.reasonCode;
   MQTTProperties_free (&connect_properties);
@@ -315,11 +367,26 @@ mqtt_connect (mqtt_t *mqtt, const char *server_uri)
 int
 mqtt_init (const char *server_uri)
 {
-  g_debug ("%s: start", __func__);
+  return mqtt_init_auth (server_uri, NULL, NULL);
+}
+/**
+ * @brief Init MQTT communication
+ *
+ * @param server_uri  Server URI
+ * @param username    Username
+ * @param password    Password
+ *
+ * @return 0 on success, <0 on error.
+ */
+int
+mqtt_init_auth (const char *server_uri, const char *username,
+                const char *password)
+{
   mqtt_t *mqtt = NULL;
 
+  g_debug ("%s: start", __func__);
+
   mqtt = g_malloc0 (sizeof (mqtt_t));
-  const char *g_server_uri;
   // Set random uuid as client id
   if (mqtt_set_client_id (mqtt) == NULL)
     {
@@ -329,10 +396,16 @@ mqtt_init (const char *server_uri)
       return -1;
     }
   g_debug ("%s: client id set: %s", __func__, mqtt->client_id);
-  g_server_uri = mqtt_get_global_server_uri ();
-  if (g_server_uri == NULL)
+  if (mqtt_get_global_server_uri () == NULL)
     mqtt_set_global_server_uri (server_uri);
-  if (mqtt_connect (mqtt, server_uri))
+
+  if (mqtt_get_global_username () == NULL)
+    mqtt_set_global_username (username);
+
+  if (mqtt_get_global_password () == NULL)
+    mqtt_set_global_password (password);
+
+  if (mqtt_connect (mqtt, server_uri, username, password))
     {
       g_warning ("%s: Unable to connect to MQTT broker.", __func__);
       g_free (mqtt);
@@ -355,6 +428,8 @@ static void
 mqtt_reinit ()
 {
   const char *server_uri;
+  const char *username;
+  const char *password;
 
   server_uri = mqtt_get_global_server_uri ();
   if (server_uri == NULL)
@@ -363,7 +438,9 @@ mqtt_reinit ()
                  "else the server URI is not set. ",
                  __func__);
     }
-  mqtt_init (server_uri);
+  username = mqtt_get_global_username ();
+  password = mqtt_get_global_password ();
+  mqtt_init_auth (server_uri, username, password);
 }
 
 /**
@@ -455,8 +532,35 @@ mqtt_publish (const char *topic, const char *msg)
 int
 mqtt_publish_single_message (const char *server_uri_in, const char *topic,
                              const char *msg)
+{
+  return mqtt_publish_single_message_auth (server_uri_in, NULL, NULL, topic,
+                                           msg);
+}
+/**
+ * @brief Send a single message with credentials
+ *
+ * This functions creates a mqtt handle, connects, sends the message, closes
+ * the connection and destroys the handler.
+ * This function should not be chosen for repeated and frequent messaging. Its
+ * meant for error messages and the likes emitted by openvas.
+ *
+ * @param server_uri_in Server URI
+ * @param username_in   Username
+ * @param passwd_in     Password
+ * @param topic         Topic to publish to
+ * @param msg           Message to publish
+ *
+ * @return 0 on success, <0 on failure.
+ */
+int
+mqtt_publish_single_message_auth (const char *server_uri_in,
+                                  const char *username_in,
+                                  const char *passwd_in, const char *topic,
+                                  const char *msg)
 {
   const char *server_uri;
+  const char *username = NULL;
+  const char *password = NULL;
   mqtt_t *mqtt = NULL;
   int ret = 0;
 
@@ -477,6 +581,17 @@ mqtt_publish_single_message (const char *server_uri_in, const char *topic,
       server_uri = server_uri_in;
     }
 
+  if (username_in == NULL || passwd_in == NULL)
+    {
+      username = mqtt_get_global_username ();
+      password = mqtt_get_global_password ();
+    }
+  else
+    {
+      username = username_in;
+      password = passwd_in;
+    }
+
   mqtt = g_malloc0 (sizeof (mqtt_t));
   // Set random uuid as client id
   if (mqtt_set_client_id (mqtt) == NULL)
@@ -485,7 +600,8 @@ mqtt_publish_single_message (const char *server_uri_in, const char *topic,
       g_free (mqtt);
       return -2;
     }
-  mqtt_connect (mqtt, server_uri);
+
+  mqtt_connect (mqtt, server_uri, username, password);
   mqtt_client_publish (mqtt, topic, msg);
 
   mqtt_disconnect (mqtt);
@@ -504,9 +620,9 @@ mqtt_publish_single_message (const char *server_uri_in, const char *topic,
  * To be able to subscribe to a topic the client needs to be connected to a
  * broker.
  *
- * @param mqtt_t	contains the mqtt client
- * @param qos	quality of service of messages within topic
- * @param topic         Topic to subscribe to
+ * @param mqtt   contains the mqtt client
+ * @param qos    quality of service of messages within topic
+ * @param topic  Topic to subscribe to
  *
  * @return 0 on success, -1 when given mqtt is not useable, -2 when subscription
  * failed.
@@ -557,8 +673,8 @@ mqtt_subscribe (const char *topic)
  *
  * This function unsubscribes given client from a given topic.
  *
- * @param mqtt_t	contains the mqtt client
- * @param topic         Topic to unsubscribe from
+ * @param mqtt   contains the mqtt client
+ * @param topic  Topic to unsubscribe from
  *
  * @return 0 on success, -1 when given mqtt is not useable, -2 when unsubscribe
  * failed.
