@@ -1,20 +1,8 @@
-/* Copyright (C) 2020-2022 Greenbone Networks GmbH
+/* SPDX-FileCopyrightText: 2020-2023 Greenbone AG
  *
- * SPDX-License-Identifier: GPL-3.0-or-later
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
+
 #include "passwordbasedauthentication.h"
 // internal usage to have access to gvm_auth initialized to verify if
 // initialization is needed
@@ -231,16 +219,28 @@ pba_verify_hash (const struct PBASettings *setting, const char *hash,
   struct crypt_data *data = NULL;
   int i = 0;
   enum pba_rc result = ERR;
+
+  char *invalid_hash = calloc (1, CRYPT_OUTPUT_SIZE);
+  memset (invalid_hash, 0, CRYPT_OUTPUT_SIZE);
+  memcpy (invalid_hash, INVALID_HASH, strlen (INVALID_HASH));
+
   if (!setting)
     goto exit;
   if (!is_prefix_supported (setting->prefix))
     goto exit;
   if (pba_is_phc_compliant (hash) != 0)
     {
+      int hash_size;
+      hash_size = hash ? strlen (hash) : strlen (invalid_hash);
+
       data = calloc (1, sizeof (struct crypt_data));
       // manipulate hash to reapply pepper
       tmp = calloc (1, CRYPT_OUTPUT_SIZE);
-      memcpy (tmp, hash ? hash : INVALID_HASH, CRYPT_OUTPUT_SIZE);
+
+      memset (tmp, 0, CRYPT_OUTPUT_SIZE);
+      memcpy (tmp, hash ? hash : invalid_hash,
+              (hash_size < CRYPT_OUTPUT_SIZE) ? hash_size
+                                              : CRYPT_OUTPUT_SIZE - 1);
       cmp = strrchr (tmp, '$');
       for (i = MAX_PEPPER_SIZE - 1; i > -1; i--)
         {
@@ -273,6 +273,7 @@ pba_verify_hash (const struct PBASettings *setting, const char *hash,
         result = INVALID;
     }
 exit:
+  free (invalid_hash);
   if (data != NULL)
     free (data);
   if (tmp != NULL)
