@@ -1,20 +1,6 @@
-/* Copyright (C) 2019-2022 Greenbone Networks GmbH
+/* SPDX-FileCopyrightText: 2019-2023 Greenbone AG
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "xmlutils.c"
@@ -499,6 +485,25 @@ Ensure (xmlutils, parse_element_free_using_child)
   element_free (element);
 }
 
+Ensure (xmlutils, print_element_to_string_prints)
+{
+  element_t element;
+  const gchar *xml;
+  GString *str;
+
+  xml = "<a aa=\"1\">a text<b><c ca=\"x\" ca2=\"y\">1</c><d/><e></e></b> and "
+        "more a text</a>";
+  str = g_string_new ("");
+
+  assert_that (parse_element (xml, &element), is_equal_to (0));
+  print_element_to_string (element, str);
+  assert_that (str->str, is_equal_to_string (
+                           "<a aa=\"1\">a text and more a text<b><c ca=\"x\" "
+                           "ca2=\"y\">1</c><d></d><e></e></b></a>"));
+  g_string_free (str, TRUE);
+  element_free (element);
+}
+
 /* Test suite. */
 
 int
@@ -530,6 +535,8 @@ main (int argc, char **argv)
   add_test_with_context (suite, xmlutils, parse_element_item_handles_cdata);
   add_test_with_context (suite, xmlutils, parse_element_free_using_child);
 
+  add_test_with_context (suite, xmlutils, print_element_to_string_prints);
+
   add_test_with_context (suite, xmlutils,
                          element_next_handles_multiple_children);
 
